import { FastifyInstance, FastifyRequest, FastifySchema, HTTPMethods as FastifyHTTPMethods } from "fastify";
import { FastifyDynamicSwaggerOptions } from "@fastify/swagger";
import { FastifySwaggerUiOptions } from "@fastify/swagger-ui";
import { TransformOptions } from "./SpecTransformer";
import { BuildJsonSchemasResult } from "./JsonSchema";
import { Models, SchemaKey, SchemaTypeOption } from "./Models";
export type RegisterOptions<S extends Models> = {
    readonly jsonSchemas: BuildJsonSchemasResult<S>;
    readonly transformSpec?: {
        readonly cache?: boolean;
        readonly routePrefix?: string;
        readonly options?: TransformOptions;
    };
    readonly swaggerOptions?: FastifyDynamicSwaggerOptions;
    readonly swaggerUiOptions?: false | FastifySwaggerUiOptions;
};
type HTTPMethods = Lowercase<FastifyHTTPMethods> & keyof FastifyInstance;
type RouteHandlerParams<M extends Models, Params extends void | SchemaKey<M>, Body extends void | SchemaKey<M>, Querystring extends void | SchemaKey<M>> = FastifyRequest<{
    Params: SchemaTypeOption<M, Params>;
    Body: SchemaTypeOption<M, Body>;
    Querystring: SchemaTypeOption<M, Querystring>;
}>;
type RouteHandler<M extends Models, Params extends void | SchemaKey<M>, Body extends void | SchemaKey<M>, Reply extends void | SchemaKey<M>, Querystring extends void | SchemaKey<M>> = (params: RouteHandlerParams<M, Params, Body, Querystring>) => Promise<SchemaTypeOption<M, Reply>>;
type RouteConfig<M extends Models = Models, Method extends HTTPMethods = HTTPMethods, Params extends void | SchemaKey<M> = void, Body extends void | SchemaKey<M> = void, Reply extends void | SchemaKey<M> = void, Querystring extends void | SchemaKey<M> = void> = {
    readonly url: string;
    readonly method: Method;
    readonly operationId: string;
    readonly description?: string;
    readonly params?: Exclude<Params, void> | {
        readonly description: string;
        readonly key: Exclude<Params, void>;
    };
    readonly body?: Exclude<Body, void> | {
        readonly description: string;
        readonly key: Exclude<Body, void>;
    };
    readonly reply?: Exclude<Reply, void> | {
        readonly description: string;
        readonly key: Exclude<Reply, void>;
    };
    readonly querystring?: Exclude<Querystring, void> | {
        readonly description: string;
        readonly key: Exclude<Querystring, void>;
    };
    readonly handler: RouteHandler<M, Params, Body, Reply, Querystring>;
} & FastifySchema;
export type FastifyZod<M extends Models> = {
    readonly [Method in HTTPMethods]: <Params extends void | SchemaKey<M> = void, Body extends void | SchemaKey<M> = void, Reply extends void | SchemaKey<M> = void, Querystring extends void | SchemaKey<M> = void>(url: string, config: Omit<RouteConfig<M, Method, Params, Body, Reply, Querystring>, `url` | `method` | `schema` | `handler`>, handler: RouteHandler<M, Params, Body, Reply, Querystring>) => void;
};
export type FastifyZodInstance<M extends Models> = FastifyInstance & {
    readonly zod: FastifyZod<M>;
};
export declare const withRefResolver: (options: FastifyDynamicSwaggerOptions) => FastifyDynamicSwaggerOptions;
export declare const register: <S extends Models>(f: FastifyInstance, { jsonSchemas: { schemas, $ref }, swaggerOptions, swaggerUiOptions, transformSpec, }: RegisterOptions<S>) => Promise<FastifyZodInstance<S>>;
export {};
